#include <QDBusConnection>
#include <QDBusMessage>

#include "dbus.hpp"
#include "log.hpp"

namespace kdk
{
namespace kabase
{

DBus::DBus() = default;

DBus::~DBus() = default;

bool DBus::registerService(QString serviceName)
{
    if (serviceName.isEmpty()) {
        error << "kabase : service is empty!";
    }

    QDBusConnection connection = QDBusConnection::sessionBus();

    if (!connection.registerService(serviceName)) {
        error << "kabase : dbus register service fail " << connection.lastError().message();
        return false;
    }

    return true;
}

bool DBus::registerObject(QString objectPath, QObject *object)
{
    if (objectPath.isEmpty() || object == nullptr) {
        error << "kabase : args error!";
        return false;
    }

    QDBusConnection connection = QDBusConnection::sessionBus();

    if (!connection.registerObject(objectPath, object, QDBusConnection::ExportAllSlots)) {
        error << "kabase : dbus register object fail " << connection.lastError().message();
        return false;
    }

    return true;
}

QList<QVariant> DBus::callMethod(QString serviceName, QString objectPath, QString interfaceName, QString methodName,
                                 QList<QVariant> args)
{
    QList<QVariant> ret;
    ret.clear();

    if (serviceName.isEmpty() || objectPath.isEmpty() || interfaceName.isEmpty() || methodName.isEmpty()) {
        error << "kabase : args error!";
        return ret;
    }

    QDBusMessage message = QDBusMessage::createMethodCall(serviceName, objectPath, interfaceName, methodName);
    if (!args.isEmpty()) {
        message.setArguments(args);
    }

    QDBusMessage reply = QDBusConnection::sessionBus().call(message);
    if (reply.type() == QDBusMessage::ReplyMessage) {
        ret = reply.arguments();
        return ret;
    } else {
        error << "kabase : dbus call method fail " << reply.errorMessage();
    }

    return ret;
}

} /* namespace kabase */
} /* namespace kdk */