#ifndef IMAGE_CONVERTER_H_
#define IMAGE_CONVERTER_H_

#include <opencv4/opencv2/core.hpp>
#include <opencv4/opencv2/imgcodecs.hpp>
#include <opencv4/opencv2/imgproc.hpp>
#include <FreeImage.h>
#include <FreeImagePlus.h>
#include <QImage>

#include "image/utils.hpp"

namespace kdk
{
namespace kabase
{

class Converter
{
public:
    Converter();
    ~Converter();

    /**
     * @brief FIBITMAP 转换 Mat
     *
     * @param fibitMap FIBITMAP数据指针
     *
     * @return 转换后的Mat
     */
    cv::Mat fibitmapToMat(FIBITMAP *fibitMap);

    /**
     * @brief Mat 转换 QImage
     *
     * @param mat Mat数据
     *
     * @return 转换后的QImage
     */
    QImage matToQimage(cv::Mat mat);

    /**
     * @brief Mat 转换 FIBITMAP
     *
     * @param mat Mat数据
     *
     * @return 转换后的FIBITMAP
     */
    FIBITMAP *matToFibitmap(cv::Mat mat);

    /**
     * @brief QImage 转换 Mat
     *
     * @param image image对象
     *
     * @return 转换后的Mat
     */
    cv::Mat qimageToMat(QImage image, image::ImageFormat format);
};

} /* namespace kabase */
} /* namespace kdk */

#endif