#include <unistd.h>

#include <QDBusMessage>

#include "log.hpp"
#include "user_manual.hpp"
#include "dbus.hpp"

namespace kdk
{
namespace kabase
{
namespace
{
constexpr char dbusServiceNamePrefix[] = "com.kylinUserGuide.hotel";
constexpr char dbusObjectPath[] = "/";
constexpr char dbusInterfaceName[] = "com.guide.hotel";
constexpr char dbusCallUserManualMethod[] = "showGuide";

const QString dbusServiceName = QString(dbusServiceNamePrefix) + QString("_") + QString::number(getuid());
} // namespace

UserManualManagement::UserManualManagement() = default;

UserManualManagement::~UserManualManagement() = default;

bool UserManualManagement::callUserManual(AppName appName)
{
    QString adapAppName = adapterAppName(Currency::getAppName(appName));
    if (adapAppName.isEmpty()) {
        error << "kabase : app name is empty!";
        return false;
    }

    QList<QVariant> args;
    args << QVariant(adapAppName);
    DBus::callMethod(dbusServiceName, dbusObjectPath, dbusInterfaceName, dbusCallUserManualMethod, args);

    return true;
}

bool UserManualManagement::callUserManual(QString appName)
{
    if (appName.isEmpty()) {
        error << "kabase : app name is empty!";
        return false;
    }

    QList<QVariant> args;
    args << QVariant(appName);
    DBus::callMethod(dbusServiceName, dbusObjectPath, dbusInterfaceName, dbusCallUserManualMethod, args);

    return true;
}

QString UserManualManagement::adapterAppName(QString appName)
{
    if (appName.isEmpty()) {
        return QString("");
    } else if (appName == QString("kylin-ipmsg")) {
        return QString("messages");
    } else {
        return appName;
    }
}

} /* namespace kabase */
} /* namespace kdk */