#include "kborderlessbutton.h"
#include "themeController.h"
#include <QStyleOptionButton>
#include <QPainter>

namespace kdk
{
class KBorderlessButtonPrivate:public QObject,public ThemeController
{
    Q_DECLARE_PUBLIC(KBorderlessButton)
    Q_OBJECT
public:
    KBorderlessButtonPrivate(KBorderlessButton*parent):
    q_ptr(parent),m_hoverd(false)
    {
        setParent(parent);
    }

protected:
    void changeTheme();
private:
    KBorderlessButton *q_ptr;
    bool m_hoverd;
};

KBorderlessButton::KBorderlessButton(QWidget* parent):
    QPushButton(parent)
    ,d_ptr(new KBorderlessButtonPrivate(this))
{
    Q_D(KBorderlessButton);
    setSizePolicy(QSizePolicy::Fixed,QSizePolicy::Fixed);
    this->setCursor(Qt::PointingHandCursor);
    setFocusPolicy(Qt::NoFocus);
}

KBorderlessButton::KBorderlessButton(const QString &text, QWidget *parent):KBorderlessButton(parent)
{
    setText(text);
}

KBorderlessButton::KBorderlessButton(const QIcon &icon, const QString &text, QWidget *parent):KBorderlessButton(parent)
{
    setIcon(icon);
    setText(text);
}

KBorderlessButton::KBorderlessButton(const QIcon &icon, QWidget *parent):KBorderlessButton(parent)
{
    setIcon(icon);
}

KBorderlessButton::~KBorderlessButton()
{
}

void KBorderlessButton::setIcon(const QIcon &icon)
{
    Q_D(KBorderlessButton);
    QPushButton::setIcon(icon);
    d->changeTheme();
}

bool KBorderlessButton::eventFilter(QObject *watched, QEvent *event)
{
    Q_D(KBorderlessButton);
    return QPushButton::eventFilter(watched,event);
}

void KBorderlessButton::paintEvent(QPaintEvent *event)
{
    Q_D(KBorderlessButton);
    QStyleOptionButton option;
    initStyleOption(&option);
    QPainter p(this);
    QColor borderColor;
    QColor fontColor;
    QColor highlight = option.palette.highlight().color();
    QColor mix = option.palette.brightText().color();
    if(!option.state.testFlag(QStyle::State_Enabled))
    {
        borderColor = option.palette.color(QPalette::Disabled,QPalette::Button);
        fontColor = option.palette.color(QPalette::Disabled,QPalette::ButtonText);
    }
    if(ThemeController::themeMode() == ThemeFlag::LightTheme){

        if(option.state.testFlag(QStyle::State_MouseOver))
        {
            if(option.state.testFlag(QStyle::State_Sunken))
            {
                borderColor = ThemeController::mixColor(highlight,mix,0.2);
                fontColor = ThemeController::mixColor(highlight,mix,0.2);
            }
            else
            {
                borderColor = ThemeController::mixColor(highlight,mix,0.1);
                fontColor = ThemeController::mixColor(highlight,mix,0.1);
            }
        }
        else
        {
            if(!this->icon().isNull()){
                fontColor = option.palette.buttonText().color();
                borderColor = option.palette.button().color();
            }
            else {
                borderColor = highlight;
                fontColor = highlight;
            }
        }
    }
    else{
        if(option.state.testFlag(QStyle::State_MouseOver))
        {
            if(option.state.testFlag(QStyle::State_Sunken))
            {
                borderColor = ThemeController::mixColor(highlight,mix,0.05);
                fontColor = ThemeController::mixColor(highlight,mix,0.05);
            }
            else
            {
                borderColor = ThemeController::mixColor(highlight,mix,0.2);
                fontColor = ThemeController::mixColor(highlight,mix,0.2);
            }
        }
        else
        {
            if(!this->icon().isNull()){
                fontColor = option.palette.buttonText().color();
                borderColor = option.palette.button().color();
            }
            else {
                borderColor = highlight;
                fontColor = highlight;
            }
        }
    }
    p.setBrush(Qt::NoBrush);
    p.setRenderHint(QPainter::HighQualityAntialiasing);
    p.setRenderHint(QPainter::Antialiasing);
    p.setRenderHint(QPainter::TextAntialiasing);
    QPen pen;
    pen.setCapStyle(Qt::RoundCap);
    pen.setJoinStyle(Qt::RoundJoin);
    pen.setWidth(1);
    pen.setColor(fontColor);
    p.setPen(pen);
    QPoint point;
    QRect ir = option.rect;
    uint tf = Qt::AlignVCenter;
    if (!option.icon.isNull()) {
        //Center both icon and text
        QIcon::Mode mode = option.state & QStyle::State_Enabled ? QIcon::Normal
                                                         : QIcon::Disabled;
        if (mode == QIcon::Normal && option.state & QStyle::State_HasFocus)
            mode = QIcon::Active;
        QIcon::State state = QIcon::Off;
        if (option.state & QStyle::State_On)
            state = QIcon::On;

        QPixmap pixmap = option.icon.pixmap(option.iconSize, mode, state);
        pixmap = ThemeController::drawColoredPixmap(this->icon().pixmap(iconSize()),fontColor);
        int w = pixmap.width() / pixmap.devicePixelRatio();
        int h = pixmap.height() / pixmap.devicePixelRatio();

        if (!option.text.isEmpty())
            w += option.fontMetrics.boundingRect(option.rect, tf, option.text).width() + 2;

        point = QPoint(ir.x() + ir.width() / 2 - w / 2,
                       ir.y() + ir.height() / 2 - h / 2);

        w = pixmap.width() / pixmap.devicePixelRatio();

        if (option.direction == Qt::RightToLeft)
            point.rx() += w;

        p.drawPixmap(this->style()->visualPos(option.direction, option.rect, point), pixmap);
        if (option.direction == Qt::RightToLeft)
            ir.translate(-point.x() - 2, 0);
        else
            ir.translate(point.x() + w + 4, 0);
        // left-align text if there is
        if (!option.text.isEmpty())
            tf |= Qt::AlignLeft;
    }
    else
    {
        tf |= Qt::AlignHCenter;
    }
    p.drawText(ir,tf,option.text);
}

QSize KBorderlessButton::sizeHint() const
{
    QFontMetrics fm(this->font());
    QSize size;
    if(this->icon().isNull())
        size = QSize(fm.width(this->text()),fm.height());
    else
    {
        int height = std::max(fm.height(),this->iconSize().height());
        int width  = fm.width(this->text()) + this->iconSize().width() + 4;
        size = QSize(width,height);
    }
    return size;
}

void KBorderlessButtonPrivate::changeTheme()
{
    Q_Q(KBorderlessButton);
    initThemeStyle();
}
}
#include "kborderlessbutton.moc"
#include "moc_kborderlessbutton.cpp"
