#include <QHBoxLayout>
#include <QMouseEvent>
#include "kiconbar.h"
#include "themeController.h"

namespace kdk
{
class KIconBarPrivate:public QObject,public ThemeController
{
    Q_OBJECT
    Q_DECLARE_PUBLIC(KIconBar)

public:
    KIconBarPrivate(KIconBar*parent);

protected:
    void changeTheme();

private:
    KIconBar* q_ptr;
    QLabel *m_pIconLabel;
    QLabel *m_pNameLabel;
    QHBoxLayout *m_pMainLayout;
    QString m_IconName;
};

KIconBar::KIconBar(QWidget*parent)
    :QFrame(parent),
      d_ptr(new KIconBarPrivate(this))
{
    setObjectName("IconBar");
    setFixedHeight(40);
}

KIconBar::KIconBar(const QString &iconName, const QString &text, QWidget *parent)
    :KIconBar(parent)
{
    setIcon(iconName);
    setWidgetName(text);
}

KIconBar::~KIconBar()
{

}

void KIconBar::setIcon(const QString &iconName)
{
    Q_D(KIconBar);

    if(iconName.isEmpty())
        return;
    if(!d->m_pIconLabel)
        return;
    d->m_IconName = iconName;
    d->m_pIconLabel->setPixmap(QIcon::fromTheme(iconName).pixmap(QSize(24,24)));
    d->m_pIconLabel->setScaledContents(true);
    setWindowIcon(QIcon::fromTheme(iconName).pixmap(QSize(36,36)));
}

void KIconBar::setIcon(const QIcon &icon)
{
    Q_D(KIconBar);

    if(!d->m_pIconLabel)
        return;
    d->m_pIconLabel->setPixmap(icon.pixmap(24,24));
    d->m_pIconLabel->setScaledContents(true);
    setWindowIcon(icon.pixmap(QSize(36,36)));
}

void KIconBar::setWidgetName(const QString &widgetName)
{
    Q_D(KIconBar);

    if(widgetName.isEmpty())
        return;
    if(!d->m_pNameLabel)
        return;
    d->m_pNameLabel->setText(widgetName);
}

QLabel *KIconBar::nameLabel()
{
    Q_D(KIconBar);
    return d->m_pNameLabel;
}

QLabel *KIconBar::iconLabel()
{
    Q_D(KIconBar);
    return d->m_pIconLabel;
}

void KIconBar::mouseDoubleClickEvent(QMouseEvent *event)
{
    Q_D(KIconBar);
    if(event->button() == Qt::LeftButton)
        Q_EMIT doubleClick();
}

KIconBarPrivate::KIconBarPrivate(KIconBar *parent)
    :q_ptr(parent)
{
    Q_Q(KIconBar);
    m_pMainLayout = new QHBoxLayout(q);
    m_pIconLabel = new QLabel(q);
    m_pNameLabel = new QLabel(q);

    m_pMainLayout->addSpacing(8);
    m_pMainLayout->addWidget(m_pIconLabel);
    m_pMainLayout->addWidget(m_pNameLabel);
    m_pMainLayout->addStretch();

    changeTheme();
    connect(m_gsetting,&QGSettings::changed,this,&KIconBarPrivate::changeTheme);
}

void KIconBarPrivate::changeTheme()
{
    initThemeStyle();
}
}
#include "kiconbar.moc"
#include "moc_kiconbar.cpp"

