#include "ktabbar.h"
#include "themeController.h"
#include <QStyleOptionTab>
#include <QPainter>
#include <QStylePainter>

namespace kdk
{
class KTabBarPrivate:public QObject,public ThemeController
{
    Q_OBJECT
    Q_DECLARE_PUBLIC(KTabBar)

public:
    KTabBarPrivate(KTabBar*parent)
        :q_ptr(parent)
    {
        setParent(parent);
    }
    void changeTheme();
private:
    KTabBar*q_ptr;
    KTabBarStyle m_kTabBarStyle;
    int m_borderRadius = 6;
    int m_horizontalMargin = 1;
    int m_topMargin = 5;
    QColor m_bkgrdColor;
};

KTabBar::KTabBar(KTabBarStyle barStyle,QWidget* parent):
    QTabBar(parent),
    d_ptr(new KTabBarPrivate(this))
{
    Q_D(KTabBar);
    d->m_borderRadius = 6;
    d->m_kTabBarStyle = barStyle;

    //this->setObjectName("KTabbar");
    d->changeTheme();
    connect(d->m_gsetting,&QGSettings::changed, d,&KTabBarPrivate::changeTheme);
}

KTabBar::~KTabBar()
{

}

void KTabBar::setTabBarStyle(KTabBarStyle barStyle)
{
    Q_D(KTabBar);
    d->m_kTabBarStyle = barStyle;
}
KTabBarStyle KTabBar::barStyle()
{
    Q_D(KTabBar);
    return d->m_kTabBarStyle;
}

void KTabBar::setBorderRadius(int radius)
{
    Q_D(KTabBar);
    if(radius < 0 || radius > 20)
        return;
    d->m_borderRadius = radius;
    update();
}

int KTabBar::borderRadius()
{
    Q_D(KTabBar);
    if(d->m_kTabBarStyle == Sliding)
        return 0;
    else
        return d->m_borderRadius;
}

void KTabBar::setBackgroundColor(const QColor &color)
{
    Q_D(KTabBar);
    d->m_bkgrdColor = color;
}

QSize KTabBar::sizeHint() const
{
    QSize size(36,36);
    return size;
}

QSize KTabBar::minimumTabSizeHint(int index) const
{
    Q_UNUSED(index)
    QSize size(100,36);
    return size;
}

void KTabBar::paintEvent(QPaintEvent *event)
{
    Q_D(KTabBar);
    QPainter p(this);

    QColor fontColor;
    QColor mix;
    for (int i = 0; i < count(); ++i)
    {
        QStyleOptionTab option;
        initStyleOption(&option, i);
        switch (d->m_kTabBarStyle)
        {
        case SegmentDark:
        {
            mix = option.palette.brightText().color();
            fontColor = option.palette.buttonText().color();
            QColor bkgrdColor = d->m_bkgrdColor;
            if(option.state.testFlag(QStyle::State_MouseOver))
            {
                if (option.state & QStyle::State_Sunken)
                    bkgrdColor = ThemeController::mixColor(bkgrdColor,mix,0.2);
                else
                    bkgrdColor = ThemeController::mixColor(bkgrdColor,mix,0.05);
            }
            if(option.state.testFlag(QStyle::State_Selected) && !option.state.testFlag(QStyle::State_Sunken))
            {
                bkgrdColor = option.palette.highlight().color();
                fontColor = QColor("#FFFFFF");
            }
            QRect rect = option.rect.adjusted(d->m_horizontalMargin,0,-d->m_horizontalMargin,-d->m_topMargin);
            if(option.position == QStyleOptionTab::Middle)
            {
                p.save();
                p.setBrush(bkgrdColor);
                p.setPen(Qt::NoPen);
                p.setRenderHint(QPainter::Antialiasing);
                //利用quadto绘制圆角矩形会出现一个像素的偏差，修正一下QRect底部高度
                p.drawRect(rect.adjusted(0,0,0,-1));
                p.restore();
                p.setBrush(Qt::NoBrush);
                p.setPen(fontColor);
                p.setRenderHint(QPainter::Antialiasing);
                QPoint point;
                uint tf = Qt::AlignVCenter;
                if (!option.icon.isNull()) {
                    QIcon::Mode mode = option.state & QStyle::State_Enabled ? QIcon::Normal
                                                                     : QIcon::Disabled;
                    if (mode == QIcon::Normal && option.state & QStyle::State_HasFocus)
                        mode = QIcon::Active;
                    QIcon::State state = QIcon::Off;
                    if (option.state & QStyle::State_On)
                        state = QIcon::On;

                    QPixmap pixmap = option.icon.pixmap(option.iconSize, mode, state);

                    int w = pixmap.width() / pixmap.devicePixelRatio();
                    int h = pixmap.height() / pixmap.devicePixelRatio();

                    if (!option.text.isEmpty())
                        w += option.fontMetrics.boundingRect(option.rect, tf, option.text).width() + 2;

                    point = QPoint(rect.x() + rect.width() / 2 - w / 2,
                                   rect.y() + rect.height() / 2 - h / 2);

                    w = pixmap.width() / pixmap.devicePixelRatio();

                    if (option.direction == Qt::RightToLeft)
                        point.rx() += w;

                    p.drawPixmap(this->style()->visualPos(option.direction, option.rect, point), pixmap);

                    if (!option.text.isEmpty()){
                        int subH = std::max(option.iconSize.height(),option.fontMetrics.height());
                        int icon_Y = (rect.height() - subH) / 2;
                        int text_X = point.x() + option.iconSize.width() + 4;
                        int text_Y = icon_Y;
                        QRect textRect(text_X, text_Y, option.fontMetrics.width(option.text), option.fontMetrics.height());
                        p.drawText(textRect,tf,option.text);
                    }
                }
                else
                {
                    tf |= Qt::AlignHCenter;
                    p.drawText(rect,tf,option.text);
                }
            }
            else if(option.position == QStyleOptionTab::Beginning)
            {
                p.save();
                p.setBrush(bkgrdColor);
                p.setPen(Qt::NoPen);
                p.setRenderHint(QPainter::Antialiasing);
                QPainterPath path;
                path.moveTo(rect.topLeft() + QPointF(0, d->m_borderRadius));
                path.lineTo(rect.bottomLeft() - QPointF(0, d->m_borderRadius));
                path.quadTo(rect.bottomLeft(), rect.bottomLeft() + QPointF(d->m_borderRadius, 0));
                path.lineTo(rect.bottomRight());
                path.lineTo(rect.topRight());
                path.lineTo(rect.topLeft() + QPointF(d->m_borderRadius, 0));
                path.quadTo(rect.topLeft(), rect.topLeft() + QPointF(0, d->m_borderRadius));
                p.drawPath(path);
                p.restore();
                p.setBrush(Qt::NoBrush);
                p.setPen(fontColor);
                p.setRenderHint(QPainter::Antialiasing);
                QPoint point;
                uint tf = Qt::AlignVCenter;
                if (!option.icon.isNull()) {
                    QIcon::Mode mode = option.state & QStyle::State_Enabled ? QIcon::Normal
                                                                     : QIcon::Disabled;
                    if (mode == QIcon::Normal && option.state & QStyle::State_HasFocus)
                        mode = QIcon::Active;
                    QIcon::State state = QIcon::Off;
                    if (option.state & QStyle::State_On)
                        state = QIcon::On;

                    QPixmap pixmap = option.icon.pixmap(option.iconSize, mode, state);

                    int w = pixmap.width() / pixmap.devicePixelRatio();
                    int h = pixmap.height() / pixmap.devicePixelRatio();

                    if (!option.text.isEmpty())
                        w += option.fontMetrics.boundingRect(option.rect, tf, option.text).width() + 2;

                    point = QPoint(rect.x() + rect.width() / 2 - w / 2,
                                   rect.y() + rect.height() / 2 - h / 2);

                    w = pixmap.width() / pixmap.devicePixelRatio();

                    if (option.direction == Qt::RightToLeft)
                        point.rx() += w;

                    p.drawPixmap(this->style()->visualPos(option.direction, option.rect, point), pixmap);

                    if (option.direction == Qt::RightToLeft)
                        rect.translate(-point.x() - 2, 0);
                    else
                        rect.translate(point.x() + w + 4, 0);

                    if (!option.text.isEmpty())
                        tf |= Qt::AlignLeft;
                }
                else
                {
                    tf |= Qt::AlignHCenter;
                }
                p.drawText(rect,tf,option.text);
            }
            else
            {
                p.save();
                p.setBrush(bkgrdColor);
                p.setPen(Qt::NoPen);
                p.setRenderHint(QPainter::Antialiasing);
                QPainterPath path;
                path.moveTo(rect.topLeft());
                path.lineTo(rect.bottomLeft());
                path.lineTo(rect.bottomRight() - QPointF(d->m_borderRadius,0));
                path.quadTo(rect.bottomRight(), rect.bottomRight() - QPointF(0, d->m_borderRadius));
                path.lineTo(rect.topRight() + QPointF(0, d->m_borderRadius));
                path.quadTo(rect.topRight(), rect.topRight() - QPointF(d->m_borderRadius,0));
                path.lineTo(rect.topLeft());
                p.drawPath(path);
                p.restore();
                p.setBrush(Qt::NoBrush);
                p.setPen(fontColor);
                p.setRenderHint(QPainter::Antialiasing);
                QPoint point;
                uint tf = Qt::AlignVCenter;
                if (!option.icon.isNull()) {

                    QIcon::Mode mode = option.state & QStyle::State_Enabled ? QIcon::Normal
                                                                     : QIcon::Disabled;
                    if (mode == QIcon::Normal && option.state & QStyle::State_HasFocus)
                        mode = QIcon::Active;
                    QIcon::State state = QIcon::Off;
                    if (option.state & QStyle::State_On)
                        state = QIcon::On;

                    QPixmap pixmap = option.icon.pixmap(option.iconSize, mode, state);

                    int w = pixmap.width() / pixmap.devicePixelRatio();
                    int h = pixmap.height() / pixmap.devicePixelRatio();

                    if (!option.text.isEmpty())
                        w += option.fontMetrics.boundingRect(option.rect, tf, option.text).width() + 4;

                    point = QPoint(rect.x() + rect.width() / 2 - w / 2,
                                   rect.y() + rect.height() / 2 - h / 2);

                    w = pixmap.width() / pixmap.devicePixelRatio();

                    if (option.direction == Qt::RightToLeft)
                        point.rx() += w;

                    p.drawPixmap(this->style()->visualPos(option.direction, option.rect, point), pixmap);

                    if (!option.text.isEmpty()){
                        int subH = std::max(option.iconSize.height(),option.fontMetrics.height());
                        int icon_Y = (rect.height() - subH) / 2;
                        int text_X = point.x() + option.iconSize.width() + 4;
                        int text_Y = icon_Y;
                        QRect textRect(text_X, text_Y, option.fontMetrics.width(option.text), option.fontMetrics.height());
                        p.drawText(textRect,tf,option.text);
                    }
                }
                else
                {
                    tf |= Qt::AlignHCenter;
                    p.drawText(rect,tf,option.text);
                }
            }
            break;
        }
        case SegmentLight:
        {
            mix = option.palette.brightText().color();
            fontColor = option.palette.buttonText().color();
            QColor bkgrdColor = d->m_bkgrdColor;
            QColor baseColor = option.palette.base().color();
            if(option.state.testFlag(QStyle::State_MouseOver))
            {
                if (option.state & QStyle::State_Sunken)
                {
                    bkgrdColor = ThemeController::mixColor(baseColor,mix,0.2);
                }
                else
                {
                    bkgrdColor = ThemeController::mixColor(baseColor,mix,0.05);
                }
            }
            if(option.state.testFlag(QStyle::State_Selected) && !option.state.testFlag(QStyle::State_Sunken))
            {
                bkgrdColor = option.palette.highlight().color();
                fontColor = QColor("#FFFFFF");
            }
            p.save();
            p.setBrush(bkgrdColor);
            p.setPen(Qt::NoPen);
            p.setRenderHint(QPainter::Antialiasing);
            p.drawRoundedRect(option.rect.adjusted(d->m_horizontalMargin,0,-d->m_horizontalMargin,-d->m_topMargin),
                              d->m_borderRadius,d->m_borderRadius);
            p.restore();
            p.setBrush(Qt::NoBrush);
            p.setPen(fontColor);
            p.setRenderHint(QPainter::Antialiasing);
            QRect rect = option.rect.adjusted(d->m_horizontalMargin,0,-d->m_horizontalMargin,-d->m_topMargin);
            QPoint point;
            uint tf = Qt::AlignVCenter;
            if (!option.icon.isNull()) {
                QIcon::Mode mode = option.state & QStyle::State_Enabled ? QIcon::Normal
                                                                 : QIcon::Disabled;
                if (mode == QIcon::Normal && option.state & QStyle::State_HasFocus)
                    mode = QIcon::Active;
                QIcon::State state = QIcon::Off;
                if (option.state & QStyle::State_On)
                    state = QIcon::On;

                QPixmap pixmap = option.icon.pixmap(option.iconSize, mode, state);

                int w = pixmap.width() / pixmap.devicePixelRatio();
                int h = pixmap.height() / pixmap.devicePixelRatio();

                if (!option.text.isEmpty())
                    w += option.fontMetrics.boundingRect(option.rect, tf, option.text).width() + 2;

                point = QPoint(rect.x() + rect.width() / 2 - w / 2,
                               rect.y() + rect.height() / 2 - h / 2);

                w = pixmap.width() / pixmap.devicePixelRatio();

                if (option.direction == Qt::RightToLeft)
                    point.rx() += w;

                p.drawPixmap(this->style()->visualPos(option.direction, option.rect, point), pixmap);

                if (!option.text.isEmpty()){
                    int subH = std::max(option.iconSize.height(),option.fontMetrics.height());
                    int icon_Y = (rect.height() - subH) / 2;
                    int text_X = point.x() + option.iconSize.width() + 4;
                    int text_Y = icon_Y;
                    QRect textRect(text_X, text_Y, option.fontMetrics.width(option.text), option.fontMetrics.height());
                    p.drawText(textRect,tf,option.text);
                }
            }
            else
            {
                tf |= Qt::AlignHCenter;
                p.drawText(rect,tf,option.text);
            }
            break;
        }
        case Sliding:
        {
            mix = option.palette.brightText().color();
            fontColor = option.palette.buttonText().color();
            QColor bkgrdColor = d->m_bkgrdColor;
            if(option.state.testFlag(QStyle::State_MouseOver))
            {
                if (option.state & QStyle::State_Sunken)
                {
                    bkgrdColor = ThemeController::mixColor(bkgrdColor,mix,0.2);
                    fontColor = ThemeController::mixColor(fontColor,mix,0.2);
                }
                else
                {
                    bkgrdColor = ThemeController::mixColor(bkgrdColor,mix,0.05);
                    fontColor = ThemeController::mixColor(fontColor,mix,0.05);
                }
            }
            if(option.state.testFlag(QStyle::State_Selected) && !option.state.testFlag(QStyle::State_Sunken))
            {
                bkgrdColor = option.palette.highlight().color();
                fontColor = option.palette.highlight().color();
            }
            p.save();
            QPen pen;
            pen.setColor(bkgrdColor);
            pen.setWidth(2);
            p.setPen(pen);
            p.setRenderHint(QPainter::Antialiasing);
            QRect rect = option.rect.adjusted(d->m_horizontalMargin,0,-d->m_horizontalMargin,-d->m_topMargin);
            p.drawLine(rect.bottomLeft(),rect.bottomRight());
            p.restore();

            p.setBrush(Qt::NoBrush);
            p.setPen(fontColor);
            p.setRenderHint(QPainter::Antialiasing);
            QPoint point;
            uint tf = Qt::AlignVCenter;
            if (!option.icon.isNull()) {
                QIcon::Mode mode = option.state & QStyle::State_Enabled ? QIcon::Normal
                                                                 : QIcon::Disabled;
                if (mode == QIcon::Normal && option.state & QStyle::State_HasFocus)
                    mode = QIcon::Active;
                QIcon::State state = QIcon::Off;
                if (option.state & QStyle::State_On)
                    state = QIcon::On;

                QPixmap pixmap = option.icon.pixmap(option.iconSize, mode, state);

                int w = pixmap.width() / pixmap.devicePixelRatio();
                int h = pixmap.height() / pixmap.devicePixelRatio();

                if (!option.text.isEmpty())
                    w += option.fontMetrics.boundingRect(option.rect, tf, option.text).width() + 2;

                point = QPoint(rect.x() + rect.width() / 2 - w / 2,
                               rect.y() + rect.height() / 2 - h / 2);

                w = pixmap.width() / pixmap.devicePixelRatio();

                if (option.direction == Qt::RightToLeft)
                    point.rx() += w;

                p.drawPixmap(this->style()->visualPos(option.direction, option.rect, point), pixmap);

                if (!option.text.isEmpty()){
                    int subH = std::max(option.iconSize.height(),option.fontMetrics.height());
                    int icon_Y = (rect.height() - subH) / 2;
                    int text_X = point.x() + option.iconSize.width() + 4;
                    int text_Y = icon_Y;
                    QRect textRect(text_X, text_Y, option.fontMetrics.width(option.text), option.fontMetrics.height());
                    p.drawText(textRect,tf,option.text);
                }
            }
            else
            {
                tf |= Qt::AlignHCenter;
                p.drawText(rect,tf,option.text);
            }
            break;
        }
        default:
            break;
        }

    }
}

void KTabBarPrivate::changeTheme()
{
    Q_Q(KTabBar);
    switch (m_kTabBarStyle) {
    case SegmentDark:
        m_bkgrdColor = q->palette().button().color();
        break;
    case SegmentLight:
        m_bkgrdColor = QColor(0,0,0,0);
        break;
    case Sliding:
        m_bkgrdColor = q->palette().button().color();
        break;
    default:
        break;
    } ;
}

}
#include "ktabbar.moc"
#include "moc_ktabbar.cpp"
