#include "waylandinterface.h"

#include <QApplication>
#include <QDebug>
#include <KWayland/Client/registry.h>
#include <KWayland/Client/surface.h>
#include <QWindow>

WaylandInterface::WaylandInterface(QObject *parent)
    :AbstractInterface(parent)
{
    //m_connection = new ConnectionThread(this);

   m_connection = KWayland::Client::ConnectionThread::fromApplication(qApp);
    //connect(m_connection,&ConnectionThread::connected,[this]{
        auto registry = new Registry(this);
        registry->create(m_connection->display());


        connect(registry, &KWayland::Client::Registry::plasmaShellAnnounced, this, [=](){
            const auto interface = registry->interface(KWayland::Client::Registry::Interface::PlasmaShell);
            if (interface.name != 0) {
                m_plasmaShell = registry->createPlasmaShell(interface.name, interface.version, this);
            }
        });

        connect(registry, &Registry::plasmaWindowManagementAnnounced, this, [=](){
            const auto interface = registry->interface(Registry::Interface::PlasmaWindowManagement);
            if (interface.name != 0) {
                m_windowManager = registry->createPlasmaWindowManagement(interface.name, interface.version, this);
            }
            if(m_windowManager)
            {
                connect(m_windowManager, &PlasmaWindowManagement::windowCreated, this, &WaylandInterface::windowCreatedProxy);
                connect(m_windowManager, &PlasmaWindowManagement::activeWindowChanged, this, [&]() noexcept {
                            auto w = m_windowManager->activeWindow();
                            /*if (!w || (w && (!m_ignoredWindows.contains(w->internalId() && !isPlasmaDesktop(w)))) )*/
                            if(w)
                            {
                                emit activeWindowChanged(w ? w->internalId() : 0);
                            }
                }, Qt::QueuedConnection);
                connect(m_windowManager, &PlasmaWindowManagement::windowCreated,this,
                    [this](PlasmaWindow *window)
                {
                    if (!m_windows.contains(window->internalId())) {
                        m_windows.insert(window->internalId(), this->requestInfo(window->internalId()));
                    }
                    if(!window->icon().isNull())
                    {
                        emit windowAdded(window->internalId());
                        return;
                    }
                    connect(window,&PlasmaWindow::iconChanged,[this,window]
                    {
                        emit windowAdded(window->internalId());
                    });

                });
            }
        });

        connect(registry, &KWayland::Client::Registry::plasmaVirtualDesktopManagementAnnounced,
                             [this, registry] (quint32 name, quint32 version) {
            m_virtualDesktopManagement = registry->createPlasmaVirtualDesktopManagement(name, version, this);
            if(m_virtualDesktopManagement)
            {
                connect(m_virtualDesktopManagement, &KWayland::Client::PlasmaVirtualDesktopManagement::desktopCreated, this,
                        [this](const QString &id, quint32 position) {
                    addDesktop(id, position);
                });

                connect(m_virtualDesktopManagement, &KWayland::Client::PlasmaVirtualDesktopManagement::desktopRemoved, this,
                        [this](const QString &id) {
                    m_desktops.removeAll(id);

                    if (m_currentDesktop == id) {
                        setCurrentDesktop(QString());
                    }
                });
            }
        });
        connect(registry, &KWayland::Client::Registry::shellAnnounced, this, [=](){
            const auto interface = registry->interface(KWayland::Client::Registry::Interface::Shell);
            if (interface.name != 0) {
                m_shell = registry->createShell(interface.name, interface.version, this);
            }
        });
        registry->setup();
        m_connection->roundtrip();
    //});
   // m_connection->initConnection();

}

WaylandInterface::~WaylandInterface()
{
}

WindowInfo WaylandInterface::requestInfo(WindowId wid)
{
    WindowInfo windowInfo;

    auto w = windowFor(wid);

    if (w) {
        if (isPlasmaDesktop(w)) {
            windowInfo.setIsValid(true);
            windowInfo.setIsPlasmaDesktop(true);
            windowInfo.setWid(wid);
            //! Window Abilities
            windowInfo.setIsClosable(false);
            windowInfo.setIsFullScreenable(false);
            windowInfo.setIsGroupable(false);
            windowInfo.setIsMaximizable(false);
            windowInfo.setIsMinimizable(false);
            windowInfo.setIsMovable(false);
            windowInfo.setIsResizable(false);
            windowInfo.setIsShadeable(false);
            windowInfo.setIsVirtualDesktopsChangeable(false);
            //! Window Abilities

        } else if (isValidWindow(w)) {
            windowInfo.setIsValid(true);
            windowInfo.setWid(wid);
            //windowInfo.setParentId(w->parentWindow() ? w->parentWindow()->internalId() : 0);
            windowInfo.setIsActive(w->isActive());
            windowInfo.setIsMinimized(w->isMinimized());
            windowInfo.setIsMaxVert(w->isMaximized());
            windowInfo.setIsMaxHoriz(w->isMaximized());
            windowInfo.setIsFullscreen(w->isFullscreen());
            windowInfo.setIsShaded(w->isShaded());
            windowInfo.setIsOnAllDesktops(w->isOnAllDesktops());
            windowInfo.setIsOnAllActivities(true);
            //windowInfo.setGeometry(w->geometry());
            windowInfo.setHasSkipTaskbar(w->skipTaskbar());
            windowInfo.setIsKeepAbove(w->isKeepAbove());
            //! Window Abilities
            windowInfo.setIsClosable(w->isCloseable());
            windowInfo.setIsFullScreenable(w->isFullscreenable());
            windowInfo.setIsMaximizable(w->isMaximizeable());
            windowInfo.setIsMinimizable(w->isMinimizeable());
            windowInfo.setIsMovable(w->isMovable());
            windowInfo.setIsResizable(w->isResizable());
            windowInfo.setIsShadeable(w->isShadeable());
            windowInfo.setIsVirtualDesktopsChangeable(w->isVirtualDesktopChangeable());
            //! Window Abilities

            //windowInfo.setDisplay(w->title());
            //windowInfo.setDesktops(w->plasmaVirtualDesktops());
            //windowInfo.setActivities(QStringList());
        }
    } else {
        windowInfo.setIsValid(false);
    }

    return windowInfo;

}

void WaylandInterface::requestActivate(WindowId wid)
{
    auto w = windowFor(wid);

    if (w) {
        w->requestActivate();
        m_connection->roundtrip();
        emit windowChanged(w->internalId());
    }
}

void WaylandInterface::requestClose(WindowId wid)
{
    auto w = windowFor(wid);

    if (w) {
        w->requestClose();
        m_connection->roundtrip();
    }
}

void WaylandInterface::requestToggleKeepAbove(WindowId wid)
{
    auto w = windowFor(wid);

    if (w) {
        w->requestToggleKeepAbove();
        m_connection->roundtrip();
        emit windowChanged(w->internalId());
    }
}

void WaylandInterface::requestToggleMinimized(WindowId wid)
{
    auto w = windowFor(wid);

    if (w) {
        w->requestToggleMinimized();
        m_connection->roundtrip();
    }
}

void WaylandInterface::requestToggleMaximized(WindowId wid)
{
    auto w = windowFor(wid);

    if (w) {
        w->requestToggleMaximized();
        m_connection->roundtrip();
    }
}

QIcon WaylandInterface::iconFor(WindowId wid)
{
    auto window = windowFor(wid);

    if (window) {
        return window->icon();
    }

    return QIcon();
}

QString WaylandInterface::titleFor(WindowId wid)
{
    auto window = windowFor(wid);

    if (window) {
        return window->title();
    }
    return QString();
}

QString WaylandInterface::windowGroupFor(WindowId wid)
{
    auto window = windowFor(wid);
    if (window)
    {
        m_connection->roundtrip();
        return window->appId();
    }
    else
        return QString();
}

quint32 WaylandInterface::pid(WindowId wid)
{
    quint32 pid = 0;
    auto window = windowFor(wid);
    if (window)
    {
        m_connection->roundtrip();
        return window->pid();
    }
    else
        return pid;
}

void WaylandInterface::showCurrentDesktop()
{
    if(m_windowManager)
    {
        m_windowManager->showDesktop();
        m_connection->roundtrip();
    }

}

void WaylandInterface::hideCurrentDesktop()
{
    if(m_windowManager)
    {
        m_windowManager->hideDesktop();
        m_connection->roundtrip();
    }
}

bool WaylandInterface::windowCanBeDragged(WindowId wid)
{
    auto w = windowFor(wid);

    if (w && isValidWindow(w)) {
        WindowInfo winfo = requestInfo(wid);
        return (winfo.isValid()
                && w->isMovable()
                && !winfo.isMinimized()
                && inCurrentDesktopActivity(winfo)
                && !winfo.isPlasmaDesktop());
    }

    return false;
}

bool WaylandInterface::windowCanBeMaximized(WindowId wid)
{
    auto w = windowFor(wid);

    if (w && isValidWindow(w)) {
        WindowInfo winfo = requestInfo(wid);
        return (winfo.isValid()
                && w->isMaximizeable()
                && !winfo.isMinimized()
                && inCurrentDesktopActivity(winfo)
                && !winfo.isPlasmaDesktop());
    }

    return false;
}

WindowId WaylandInterface::activeWindow()
{
    if (!m_windowManager) {
        return 0;
    }
    m_connection->roundtrip();
    auto wid = m_windowManager->activeWindow();
    return wid ? QVariant(wid->internalId()) : 0;
}

void WaylandInterface::setGeometry(QWindow *window, const QRect &rect)
{
    if(!window)
        return;
    if (!m_plasmaShell)
        return;

    auto surface = KWayland::Client::Surface::fromWindow(window);
    if (!surface)
        return;

    auto plasmaShellSurface = m_plasmaShell->createSurface(surface, window);
    if (!plasmaShellSurface)
        return;

    plasmaShellSurface->setPosition(rect.topLeft());
    window->resize(rect.size());
}

NET::WindowType WaylandInterface::windowType(WindowId wid)
{
    return NET::WindowType::Normal;
}

void WaylandInterface::updateWindow()
{
    PlasmaWindow* w = qobject_cast<PlasmaWindow*>(QObject::sender());
    if(w && !isPlasmaPanel(w))
    {
        m_connection->roundtrip();
        emit windowChanged(w->internalId());
    }
}

void WaylandInterface::windowUnmapped()
{
    PlasmaWindow *pW = qobject_cast<PlasmaWindow*>(QObject::sender());

    if (pW) {
        untrackWindow(pW);
        emit windowRemoved(pW->internalId());
    }
}

PlasmaWindow *WaylandInterface::windowFor(WindowId wid)
{
    auto it = std::find_if(m_windowManager->windows().constBegin(), m_windowManager->windows().constEnd(), [&wid](PlasmaWindow * w) noexcept {
            return w->isValid() && w->internalId() == wid;
});

    if (it == m_windowManager->windows().constEnd()) {
        return nullptr;
    }

    return *it;
}

bool WaylandInterface::isValidWindow(const PlasmaWindow *w)
{
    return w->isValid();
}

bool WaylandInterface::isPlasmaDesktop(const PlasmaWindow *w)
{
    if (!w || (w->appId() != QLatin1String("org.kde.plasmashell"))) {
        return false;
    }

    return AbstractInterface::isPlasmaDesktop(w->geometry());
}

bool WaylandInterface::isPlasmaPanel(const PlasmaWindow *w)
{
    if(w && w->appId() == QLatin1String("ukui-panel"))
        return true;
    else
        return false;
}

void WaylandInterface::windowCreatedProxy(PlasmaWindow *w)
{
    if (!isValidWindow(w)) {
        return;
    }

    if ((w->appId() == QLatin1String("org.kde.plasmashell")) && isPlasmaPanel(w))
    {
        //registerPlasmaPanel(w->internalId());
    }
    else
    {
        trackWindow(w);
        //emit windowAdded(w->internalId());
    }

//    if (w->appId() == "latte-dock") {
//        emit latteWindowAdded();
//    }
}

void WaylandInterface::trackWindow(PlasmaWindow *w)
{
    if (!w) {
        return;
    }
    //监控activeChanged会导致最小化状态获取不准确的问题
    //connect(w, &PlasmaWindow::activeChanged, this, &WaylandInterface::updateWindow);
    connect(w, &PlasmaWindow::titleChanged, this, &WaylandInterface::updateWindow);
    connect(w, &PlasmaWindow::fullscreenChanged, this, &WaylandInterface::updateWindow);
    connect(w, &PlasmaWindow::geometryChanged, this, &WaylandInterface::updateWindow);
    connect(w, &PlasmaWindow::maximizedChanged, this, &WaylandInterface::updateWindow);
    connect(w, &PlasmaWindow::minimizedChanged, this, &WaylandInterface::updateWindow);
    connect(w, &PlasmaWindow::shadedChanged, this, &WaylandInterface::updateWindow);
    connect(w, &PlasmaWindow::skipTaskbarChanged, this, &WaylandInterface::updateWindow);
    connect(w, &PlasmaWindow::onAllDesktopsChanged, this, &WaylandInterface::updateWindow);
    connect(w, &PlasmaWindow::parentWindowChanged, this, &WaylandInterface::updateWindow);
    connect(w, &PlasmaWindow::iconChanged, this, &WaylandInterface::updateWindow);
    connect(w, &PlasmaWindow::plasmaVirtualDesktopEntered, this, &WaylandInterface::updateWindow);
    connect(w, &PlasmaWindow::plasmaVirtualDesktopLeft, this, &WaylandInterface::updateWindow);

    connect(w, &PlasmaWindow::unmapped, this, &WaylandInterface::windowUnmapped);
}

void WaylandInterface::untrackWindow(PlasmaWindow *w)
{
    if (!w) {
        return;
    }

    disconnect(w, &PlasmaWindow::activeChanged, this, &WaylandInterface::updateWindow);
    disconnect(w, &PlasmaWindow::titleChanged, this, &WaylandInterface::updateWindow);
    disconnect(w, &PlasmaWindow::fullscreenChanged, this, &WaylandInterface::updateWindow);
    disconnect(w, &PlasmaWindow::geometryChanged, this, &WaylandInterface::updateWindow);
    disconnect(w, &PlasmaWindow::maximizedChanged, this, &WaylandInterface::updateWindow);
    disconnect(w, &PlasmaWindow::minimizedChanged, this, &WaylandInterface::updateWindow);
    disconnect(w, &PlasmaWindow::shadedChanged, this, &WaylandInterface::updateWindow);
    disconnect(w, &PlasmaWindow::skipTaskbarChanged, this, &WaylandInterface::updateWindow);
    disconnect(w, &PlasmaWindow::onAllDesktopsChanged, this, &WaylandInterface::updateWindow);
    disconnect(w, &PlasmaWindow::parentWindowChanged, this, &WaylandInterface::updateWindow);

    disconnect(w, &PlasmaWindow::plasmaVirtualDesktopEntered, this, &WaylandInterface::updateWindow);
    disconnect(w, &PlasmaWindow::plasmaVirtualDesktopLeft, this, &WaylandInterface::updateWindow);

    disconnect(w, &PlasmaWindow::unmapped, this, &WaylandInterface::windowUnmapped);
}

void WaylandInterface::setCurrentDesktop(QString desktop)
{
    if (m_currentDesktop == desktop) {
        return;
    }

    m_currentDesktop = desktop;
    emit currentDesktopChanged();
}

void WaylandInterface::addDesktop(const QString &id, quint32 position)
{
    if (m_desktops.contains(id)) {
        return;
    }

    m_desktops.append(id);

    const KWayland::Client::PlasmaVirtualDesktop *desktop = m_virtualDesktopManagement->getVirtualDesktop(id);

    QObject::connect(desktop, &KWayland::Client::PlasmaVirtualDesktop::activated, this,
                     [desktop, this]() {
        setCurrentDesktop(desktop->id());
    }
    );

    if (desktop->isActive()) {
        setCurrentDesktop(id);
    }
}
